@@ -16,10 +16,14 @@ setMethod("dim<-", signature(x = "mpfr", value = "ANY"),
 		  stop("invalid 'value' ( = RHS = right hand side)")
 	  })
 
-mpfrArray <- function(x, precBits, dim = length(x), dimnames = NULL)
+mpfrArray <- function(x, precBits, dim = length(x), dimnames = NULL,
+                      rnd.mode = c('N','D','U','Z'))
 {
     dim <- as.integer(dim)
-    ml <- .Call("d2mpfr1_list", x, precBits, PACKAGE="Rmpfr")
+    rnd.mode <- toupper(rnd.mode)
+    rnd.mode <- match.arg(rnd.mode)
+
+    ml <- .Call("d2mpfr1_list", x, precBits, rnd.mode, PACKAGE="Rmpfr")
     vl <- prod(dim)
     if (length(x) != vl) {
         if (vl > .Machine$integer.max)
@@ -80,10 +84,10 @@ setMethod("aperm", signature(a="mpfrArray"),
 	  })
 
 
+## `` drop the  dim() part '' :
 setMethod("as.vector", "mpfrArray", function(x) as(x, "mpfr"))
-## a "vector" in  *one* sense at least, but *not* this one,
-## and we should *not* define this ("mpfr" does not extend "vector"!):
-## setAs("mpfrArray", "vector", function(from) as(from, "mpfr")
+## a "vector" in  *one* sense at least, and "mpfr" does extend "vector":
+setAs("mpfrArray", "vector", function(from) as(from, "mpfr"))
 
 toNum <- function(from) {
     structure(.Call("mpfr2d", from, PACKAGE="Rmpfr"),
@@ -95,6 +99,11 @@ setAs("mpfrArray", "array", toNum)
 
 setAs("mpfrMatrix", "matrix", toNum)
 
+setAs("mpfrArray", "matrix", function(from) {
+    if(length(dim(from)) != 2)
+	stop("dim(.) != 2  ==> cannot be coerced to 'matrix'")
+    toNum(from)
+})
 
 print.mpfrArray <- function(x, digits = NULL, drop0trailing = FALSE, ...) {
 ##                                                            -----
@@ -133,11 +142,11 @@ setMethod(show, "mpfrArray", function(object) print.mpfrArray(object))
 .matmult.R <- function(x,y, op = 0)
 {
     if(!(is.numeric(x) || is(x,"mpfr")))
-        stop("'x' must be numeric of mpfr(Matrix)")
+	stop("'x' must be numeric or mpfr(Matrix)")
     sym <- missing(y)
     if (sym && (op > 0)) y <- x
     else if(!(is.numeric(y) || is(y,"mpfr")))
-        stop("'y' must be numeric of mpfr(Matrix)")
+	stop("'y' must be numeric or mpfr(Matrix)")
     ldx <- length(dx <- dim(x))
     ldy <- length(dy <- dim(y))
     ## "copy, paste & modify" from  do_matprod():
@@ -164,12 +173,22 @@ setMethod(show, "mpfrArray", function(object) print.mpfrArray(object))
 		ncx <- 1L # == nry
 	    }
 	}
-	else { #* crossprod */
+	else if (op == 1) { #* crossprod
 	    if (length(x) == nry) {	#* x is a col vector */
 		nrx <- nry # = length(x)
 		ncx <- 1L
 	    }
 	}
+	else { # op == 2: tcrossprod
+	    if (length(x) == ncy) {	#* x as row vector */
+		nrx <- 1L
+		ncx <- ncy # == length(x)
+	    }
+	    else if (ncy == 1) {	#* x as col vector */
+		nrx <- length(x)
+		ncx <- 1L # == ncy
+	    }
+	}
     }
     else if (ldy != 2) {		#* y not a matrix */
 	nrx <- dx[1]
@@ -185,12 +204,16 @@ setMethod(show, "mpfrArray", function(object) print.mpfrArray(object))
 		ncy <- length(y)
 	    }
 	}
-	else {
+	else if (op == 1) { #* crossprod
 	    if (length(y) == nrx) {	#* y is a col vector */
 		nry <- nrx # = length(y)
 		ncy <- 1L
 	    }
 	}
+	else { # op == 2: tcrossprod	y is a col vector
+	    nry <- length(y)
+	    ncy <- 1L
+	}
     }
     else {				#* x and y matrices */
 	nrx <- dx[1]
@@ -206,85 +229,98 @@ setMethod(show, "mpfrArray", function(object) print.mpfrArray(object))
     if (op == 0) { ## %*%
 	if (ncx != nry) stop("non-conformable arguments")
 
-        z@Dim <- c(nrx, ncy)
-        z@.Data <- vector("list", nrx*ncy)
-        if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
-            for(i in 1:nrx)
-                for (k in 0L:(ncy - 1L)) {
-                    sum <- 0
-                    for (j in 0L:(ncx - 1L))
-                        sum <- sum + x[i + j * nrx] * y[1L+ j + k * nry]
-                    z[i + k * nrx] <- sum
-                }
-        else        #/* zero-extent operations should return zeroes */
-            for(i in seq_len(nrx*ncy)) z[i] <- z0
+	z@Dim <- c(nrx, ncy)
+	z@.Data <- vector("list", nrx*ncy)
+	if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
+	    for(i in 1:nrx) {
+		j <- 0L:(ncx - 1L)
+		for (k in 0L:(ncy - 1L))
+		    z[i + k * nrx] <- sum(x[i + j * nrx] * y[1L+ j + k * nry])
+	    }
+	else	    #/* zero-extent operations should return zeroes */
+	    for(i in seq_len(nrx*ncy)) z[i] <- z0
     }
     else if (op == 1) { ## crossprod() :  x' %*% y
 	if (nrx != nry) stop("non-conformable arguments")
 
-        z@Dim <- c(ncx, ncy)
-        z@.Data <- vector("list", ncx*ncy)
-        if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
-            for(i in 0L:(ncx - 1L))
-                for (k in 0L:(ncy - 1L)) {
-                    sum <- 0
-                    for (j in 1L:nrx)
-                        sum <- sum + x[j + i * nrx] * y[j + k * nry]
-                    z[1L +i + k * ncx] <- sum
-                }
-        else
-            for(i in seq_len(ncx*ncy)) z[i] <- z0
-
+	z@Dim <- c(ncx, ncy)
+	z@.Data <- vector("list", ncx*ncy)
+	if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
+	    for(i in 0L:(ncx - 1L)) {
+		j <- 1L:nrx
+		for (k in 0L:(ncy - 1L))
+		    z[1L +i + k * ncx] <- sum(x[j + i * nrx] * y[j + k * nry])
+	    }
+	else
+	    for(i in seq_len(ncx*ncy)) z[i] <- z0
     }
-    else { ## op == 2 :  tcrossprod() :  x %*% y'
+    else { ## op == 2 :	 tcrossprod() :	 x %*% y'
 	if (ncx != ncy) stop("non-conformable arguments")
 
-        z@Dim <- c(nrx, nry)
-        z@.Data <- vector("list", nrx*nry)
-        if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
-            for(i in seq_len(nrx))
-                for (k in 0L:(nry - 1L)) {
-                    sum <- 0
-                    for (j in 0L:(ncx - 1L))
-                        sum <- sum + x[i + j * nrx] * y[1L +k + j * nry]
-                    z[i + k * nrx] <- sum
-                }
-        else
-            for(i in seq_len(nrx*nry)) z[i] <- z0
-
+	z@Dim <- c(nrx, nry)
+	z@.Data <- vector("list", nrx*nry)
+	if (nrx > 0 && ncx > 0 && nry > 0 && ncy > 0)
+	    for(i in seq_len(nrx)) {
+		j <- 0L:(ncx - 1L)
+		for (k in 0L:(nry - 1L))
+		    z[i + k * nrx] <- sum(x[i + j * nrx] * y[1L +k + j * nry])
+	    }
+	else
+	    for(i in seq_len(nrx*nry)) z[i] <- z0
     }
     z
-}
-
-setMethod("%*%", signature(x = "mpfrMatrix", y = "mpfrMatrix"),
-          function(x,y) .matmult.R(x,y, op= 0))
+} ## .matmult.R()
 
 ## "FIXME"?  make working also with "Matrix" class matrices ..
-setMethod("%*%", signature(x = "mpfrMatrix", y = "array_or_vector"),
-          function(x,y) .matmult.R(x,y, op= 0))
-setMethod("%*%", signature(x = "array_or_vector", y = "mpfrMatrix"),
-          function(x,y) .matmult.R(x,y, op= 0))
-## Even these (vectors!) -- if that's ok, drop the above two --
-## FIXME --- ditto for  crossprod / tcrossprod :
-setMethod("%*%", signature(x = "mpfr", y = "array_or_vector"),
-          function(x,y) .matmult.R(x,y, op= 0))
-setMethod("%*%", signature(x = "array_or_vector", y = "mpfr"),
-          function(x,y) .matmult.R(x,y, op= 0))
+##                             ----------------------------
+
+
+setMethod("%*%", signature(x = "mpfrMatrix", y = "mpfrMatrix"),
+	  function(x,y) .matmult.R(x,y, op= 0))
+setMethod("%*%", signature(x = "mpfrMatrix", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 0))
+setMethod("%*%", signature(x = "mpfr", y = "mpfrMatrix"),
+	  function(x,y) .matmult.R(x,y, op= 0))
+setMethod("%*%", signature(x = "mpfr", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 0))
+## These cover vectors, etc (!) :
+setMethod("%*%", signature(x = "mpfr", y = "Mnumber"),
+	  function(x,y) .matmult.R(x,y, op= 0))
+setMethod("%*%", signature(x = "Mnumber", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 0))
 
 
 setMethod("crossprod", signature(x = "mpfrMatrix", y = "mpfrMatrix"),
-          function(x,y) .matmult.R(x,y, op= 1))
-setMethod("crossprod", signature(x = "mpfr", y = "array_or_vector"),
-          function(x,y) .matmult.R(x,y, op= 1))
-setMethod("crossprod", signature(x = "array_or_vector", y = "mpfr"),
-          function(x,y) .matmult.R(x,y, op= 1))
+	  function(x,y) .matmult.R(x,y, op= 1))
+setMethod("crossprod", signature(x = "mpfrMatrix", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 1))
+setMethod("crossprod", signature(x = "mpfr", y = "mpfrMatrix"),
+	  function(x,y) .matmult.R(x,y, op= 1))
+setMethod("crossprod", signature(x = "mpfr", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 1))
+setMethod("crossprod", signature(x = "mpfr", y = "Mnumber"),
+	  function(x,y) .matmult.R(x,y, op= 1))
+setMethod("crossprod", signature(x = "Mnumber", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 1))
+## one argument-case:
+setMethod("crossprod", signature(x = "mpfr", y = "missing"),
+	  function(x,y) .matmult.R(x,x, op= 1))
 
 setMethod("tcrossprod", signature(x = "mpfrMatrix", y = "mpfrMatrix"),
-          function(x,y) .matmult.R(x,y, op= 2))
-setMethod("tcrossprod", signature(x = "mpfr", y = "array_or_vector"),
-          function(x,y) .matmult.R(x,y, op= 2))
-setMethod("tcrossprod", signature(x = "array_or_vector", y = "mpfr"),
-          function(x,y) .matmult.R(x,y, op= 2))
+	  function(x,y) .matmult.R(x,y, op= 2))
+setMethod("tcrossprod", signature(x = "mpfrMatrix", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 2))
+setMethod("tcrossprod", signature(x = "mpfr", y = "mpfrMatrix"),
+	  function(x,y) .matmult.R(x,y, op= 2))
+setMethod("tcrossprod", signature(x = "mpfr", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 2))
+setMethod("tcrossprod", signature(x = "mpfr", y = "Mnumber"),
+	  function(x,y) .matmult.R(x,y, op= 2))
+setMethod("tcrossprod", signature(x = "Mnumber", y = "mpfr"),
+	  function(x,y) .matmult.R(x,y, op= 2))
+## one argument-case:
+setMethod("tcrossprod", signature(x = "mpfr", y = "missing"),
+	  function(x,y) .matmult.R(x,x, op= 2))
 
 
 .mpfrA.subset <- function(x,i,j, ..., drop) {
@@ -326,55 +362,59 @@ setMethod("[", signature(x = "mpfrArray", i = "ANY", j = "missing",
           .mpfrA.subset)
 
 
-
-.mA.subAssign <- function(x,i,j,..., value)
+.mA.subAssign <- function(x,i,j,..., value, n.a, isMpfr)
 {
-    nA <- nargs()
-    if(nA >= 4) {
+    ## n.a :=== nargs() -- in the calling "[<-" method --
+    r <- x@.Data
+    if(n.a >= 4) {
 	## A[i,j]  /  A[i,]  /	A[,j]	but not A[i]
-	## A[i,j,k] <- v : nA == 5
-	r <- x@.Data
+	## A[i,j,k] <- v : n.a == 5
 	dim(r) <- dim(x)
 	dimnames(r) <- dimnames(x)
-	vD <- as(value, "mpfr")@.Data
-	if(nA == 4) {
-
+	if(!isMpfr)
+	   value <- mpfr(value, precBits =
+			 pmax(getPrec(value),
+			      .getPrec(if(n.a == 4) r[i,j] else r[i,j, ...]))
+			 )
+	vD <- value@.Data
+	if(n.a == 4) {
 	    r[i,j] <- vD
-
-	} else { ## nA >= 5
-
+	} else { ## n.a >= 5
 	    r[i, j, ...] <- vD
-
 	}
 	attributes(r) <- NULL
 	x@.Data <- r
     }
-    else if(nA == 3) { ##  A [ i ] <- v
-
+    else if(n.a == 3) { ##  A [ i ] <- v
+	if(!isMpfr)
+	    value <- mpfr(value, precBits = pmax(getPrec(value), .getPrec(r[i])))
 	x@.Data[i] <- value
 
-    } else { ## nA <= 2
+    } else { ## n.a <= 2
 	stop(sprintf("nargs() == %d  mpfrArray[i,j] <- value  IMPOSSIBLE?",
-		     nA))
+		     n.a))
     }
     x
 }## .mA.subAssign
 
 ## "[<-" :
-setReplaceMethod("[", signature(x = "mpfrArray", i = "ANY", j = "ANY",
-				value = "ANY"),
-		 .mA.subAssign)
+## -------
 ## E.g., for A[1,,2] <- V
 ## these are to trigger before the  ("mpfr", i,j, "mpfr")  [ ./mpfr.R ] does
-setReplaceMethod("[", signature(x = "mpfrArray", i = "ANY", j = "missing",
-				value = "mpfr"),
-		 .mA.subAssign)
-setReplaceMethod("[", signature(x = "mpfrArray", i = "missing", j = "ANY",
-				value = "mpfr"),
-		 .mA.subAssign)
-setReplaceMethod("[", signature(x = "mpfrArray", i = "ANY", j = "ANY",
-				value = "mpfr"),
-		 .mA.subAssign)
+for(it in c("ANY", "missing"))
+    for(jt in c("ANY", "missing"))
+    setReplaceMethod("[", signature(x = "mpfrArray", i = it, j = jt, value = "mpfr"),
+		     function(x,i,j,..., value)
+		     .mA.subAssign(x,i=i,j=j,...,value=value,
+				   n.a=nargs(), isMpfr = TRUE))
+## non-"mpfr" value
+for(it in c("ANY", "missing"))
+    for(jt in c("ANY", "missing"))
+    setReplaceMethod("[", signature(x = "mpfrArray", i = it, j = jt, value = "ANY"),
+		     function(x,i,j, ..., value)
+		     .mA.subAssign(x,i=i,j=j,...,value=value,
+				   n.a=nargs(), isMpfr = FALSE))
+rm(it,jt)
 
 ###-----------
 
