@@ -1,32 +1,32 @@
 #### All  coercion methods for the  "Rmpfr" classes
 
 
+
 ## was  toMpfr <-
-mpfr <- function(x, precBits, base = 10)
+mpfr <- function(x, precBits, base = 10, rnd.mode = c('N','D','U','Z'))
 {
     if(is.character(x))
 	stopifnot(length(base) == 1, 2 <= base, base <= 36)
     if(missing(precBits)) {
-	if(is.character(x)) ## number of digits --> number of bits
-	    precBits <- ceiling(log2(base) * nchar(gsub("[-.]", '', x)))
-	else if(is.logical(x))
-	    precBits <- 2L # even 1 would suffice - but need 2 (in C ?)
-	else if(is.raw(x))
-	    precBits <- 8L
-	else stop("must specify 'precBits' for numeric 'x'")
+	precBits <- getPrec(x, base = base, doNumeric = FALSE)
     }
     ## libmpfr would exit (after good error message) for precBits == 1
-    stopifnot(precBits >= 2)
+    stopifnot(precBits >= 2, is.character(rnd.mode))
+    rnd.mode <- toupper(rnd.mode)
+    rnd.mode <- match.arg(rnd.mode)
+
     if(is.numeric(x) || is.logical(x) || is.raw(x)) {
-	new("mpfr", .Call("d2mpfr1_list", x, precBits, PACKAGE="Rmpfr"))
+	new("mpfr", .Call("d2mpfr1_list", x, precBits, rnd.mode, PACKAGE="Rmpfr"))
     } else if(is.character(x)) {
-	new("mpfr", .Call("str2mpfr1_list", x, precBits, base, PACKAGE="Rmpfr"))
+	new("mpfr", .Call("str2mpfr1_list", x, precBits,
+			  base, rnd.mode, PACKAGE="Rmpfr"))
     }
     else stop("invalid 'x'. Must be numeric (logical, raw) or character")
 }
 
 setAs("numeric", "mpfr1", ## use default precision of 128 bits
-      function(from) .Call("d2mpfr1", from, 128L, PACKAGE="Rmpfr"))
+      function(from) .Call("d2mpfr1", from, 128L, "N",# <- round to [N]earest
+                           PACKAGE="Rmpfr"))
 setAs("numeric", "mpfr", function(from) mpfr(from, 128L))
 setAs("integer", "mpfr", function(from) mpfr(from,  32L))
 setAs("raw",     "mpfr", function(from) mpfr(from,   8L))
@@ -53,23 +53,23 @@ setAs("mpfr", "mpfr1", function(from) {
 
 .mpfr2str <- function(x, digits = NULL) {
     stopifnot(is.null(digits) ||
-	      (is.numeric(digits) && digits >= 0))
+	      (is.numeric(digits) && digits >= 1))
     ##	digits = NULL : use as many digits "as needed"
     .Call("mpfr2str", x, digits, PACKAGE="Rmpfr")
 }
 
 formatMpfr <-
     function(x, digits = NULL, trim = FALSE, scientific = NA,
-             big.mark = "", big.interval = 3L,
-             small.mark = "", small.interval = 5L, decimal.mark = ".",
+	     showNeg0 = TRUE,
+	     big.mark = "", big.interval = 3L,
+	     small.mark = "", small.interval = 5L, decimal.mark = ".",
 	     zero.print = NULL, drop0trailing = FALSE, ...)
 {
     stopifnot(is.null(digits) ||
-	      (is.numeric(digits) && digits >= 0))
+	      (is.numeric(digits) && digits >= 1))
     ##	digits = NULL : use as many digits "as needed"
 
     ff <- .mpfr2str(x, digits)
-    hasMinus <- sign(x) == -1
     isNum <- ff$finite	## ff$finite == is.finite(x)
     i0 <- ff$is.0	## == mpfr.is.0(x)
     ex <- ff$exp ## the *decimal* exp : one too large *unless* x == 0
@@ -77,40 +77,53 @@ formatMpfr <-
     if(is.null(digits)) digits <- nchar(r)
 
     if(any(i0)) {
+	## sign(x) == -1 "fails" for '-0'
+	hasMinus <- substr(ff$str, 1L,1L) == "-"
+	if(showNeg0) {
+	} else if(any(iN0 <- hasMinus & i0)) {
+	    ## get rid of "-" for "negative zero"
+	    r[iN0] <- substring(r[iN0], 2)
+	    hasMinus[iN0] <- FALSE
+	}
 	Ex <- ex
 	Ex[!i0] <- ex[!i0] - 1L
-    } else Ex <- ex - 1L
+    } else {
+	Ex <- ex - 1L
+	hasMinus <- sign(x) == -1
+    }
 
     if(!all(isNum))
-        r[!isNum] <- gsub("@", '', r[!isNum], fixed=TRUE)
+	r[!isNum] <- gsub("@", '', r[!isNum], fixed=TRUE)
 
     ## (maybe) add decimal point
     patch <- function(str, k)
-	paste(substr   (str, 1, k),
-	      substring(str, k+1), sep = decimal.mark)
+	paste(substr   (str, 1L, k),
+	      substring(str, k+1L), sep = decimal.mark)
 
     if(is.na(scientific))
-        scientific <- as.numeric(getOption("scipen"))
+	scientific <- as.numeric(getOption("scipen"))
     ## This very much depends on the desired format.
     ## if(scientific) --> all get a final "e<exp>"; otherwise, we
     ## adopt the following simple scheme :
     hasE <- { if(is.logical(scientific)) scientific else
-              isNum & (Ex < -4 + scientific | Ex >= digits) }
+	      isNum & (Ex < -4 + scientific | Ex >= digits) }
 
     if(any(hasE)) {
-	i. <- 1+hasMinus
+	i. <- 1L + hasMinus
 	ii <- isNum & hasE
 	r[ii] <- patch(r[ii], i.[ii])
 	## FIXME : if this is correct, make it simpler :
 	## r[hasE] <- paste(r[hasE], as.character(Ex[hasE]), sep = "e")
 	r[ii] <- paste(r[ii], as.character(Ex[ii]), sep = "e")
     }
-    if(!all(hasE)) { ## "non-scientific" i.e. with out final  e<nn> :
+    if(!all(hasE)) { ## "non-scientific" i.e. without final  e<nn> :
 	ii <- isNum & !hasE
-	iNeg <- ex <= 0	 & ii ## i.e., ex	 in {0,-1,-2,-3}
-	iPos <- ex > 0	 & ii ## i.e., ex	 in {1,2..., digits}
+	## iNeg <- ex <= 0	 & ii ## i.e., ex	 in {0,-1,-2,-3}
+	## iPos <- ex > 0	 & ii ## i.e., ex	 in {1,2..., digits}
+	iNeg <- Ex <  0	 & ii ## i.e., ex	 in {0,-1,-2,-3}
+	iPos <- Ex >= 0	 & ii ## i.e., ex	 in {1,2..., digits}
 
-	nZeros <- function(n)
+	nZeros <- function(n) ## e.g.  nZeros(2:0) gives  c("00","0", "")
 	    sapply(n, function(k) paste(rep.int("0", k), collapse = ""))
 	if(any(iNeg)) { ## "0.00..." : be careful with minus sign
 	    if(any(isMin <- hasMinus[iNeg])) {
@@ -124,14 +137,14 @@ formatMpfr <-
 	    }
 	}
 	if(any(iPos)) ## "xy.nnnn" :
-	    r[iPos] <- patch(r[iPos], (hasMinus + ex)[iPos])
+	    r[iPos] <- patch(r[iPos], (hasMinus + Ex+1L)[iPos])
     }
     prettyNum(r, big.mark = big.mark, big.interval = big.interval,
-              small.mark = small.mark,
-              small.interval = small.interval,
-              decimal.mark = decimal.mark,
-              zero.print = zero.print, drop0trailing = drop0trailing,
-              preserve.width = if (trim) "individual" else "common")
+	      small.mark = small.mark,
+	      small.interval = small.interval,
+	      decimal.mark = decimal.mark,
+	      zero.print = zero.print, drop0trailing = drop0trailing,
+	      preserve.width = if (trim) "individual" else "common")
 }
 setMethod("format", "mpfr", formatMpfr)
 
