@@ -77,8 +77,12 @@ SEXP Math_mpfr(SEXP x, SEXP op)
 	case 104: mpfr_zeta(R_i, R_i, GMP_RNDN); break;
 
 	case 106: mpfr_eint(R_i, R_i, GMP_RNDN); break;
+#if (MPFR_VERSION < MPFR_VERSION_NUM(2,4,0))
+	case 107: error("Li2() not implemented in oldish MPFR library version '%s'",
+			MPFR_VERSION_STRING);
+#else
 	case 107: mpfr_li2 (R_i, R_i, GMP_RNDN); break;
-
+#endif
 	case 111: mpfr_j0(R_i, R_i, GMP_RNDN); break;
 	case 112: mpfr_j1(R_i, R_i, GMP_RNDN); break;
 	case 113: mpfr_y0(R_i, R_i, GMP_RNDN); break;
